#include "../stdafx.h"
#include <stdlib.h>
#include "../cphidget.h"
#include "phidget_jni.h"
#include "cphidgetlist.h"
#include "com_phidgets_usb_Manager.h"
#include "com_phidgets_usb_Phidget.h"

static jclass usb_manager_class;
static jmethodID usb_manager_getPhidgetList;

int
com_phidgets_usb_Manager_OnLoad(JNIEnv *env)
{
	//USB Phidget
	if (!(usb_manager_class = (*env)->FindClass(env, "com/phidgets/usb/Manager")))
		return PFALSE;
	if (!(usb_manager_class = (jclass)(*env)->NewGlobalRef(env, usb_manager_class)))
		return PFALSE;
	
	//Methods
	if (!(usb_manager_getPhidgetList = (*env)->GetStaticMethodID(env, usb_manager_class, "getPhidgetList", "()[Lcom/phidgets/usb/Phidget;")))
		JNI_ABORT_STDERR("Couldn't get method ID getPhidgetList from usb_manager_class");
	
	return PTRUE;
}

JNIEXPORT jboolean JNICALL
Java_com_phidgets_usb_Manager_getInitialized(JNIEnv *env, jclass cls)
{
	//return AndroidUsbInitialized;
	return ANDROID_USB_GOOD;
}

JNIEXPORT void JNICALL
Java_com_phidgets_usb_Manager_setInitialized(JNIEnv *env, jclass cls, jboolean status)
{
	AndroidUsbInitialized = status;
}

//TODO: Make sure the phidgets in this list are freed at some point!!
int CPhidgetAndroidManager_getPhidgetList(CPhidgetList **curList)
{
	JNIEnv *env;
	jobjectArray phidgetList;
	jint result;
	int res = EPHIDGET_OK;
	
	CPhidgetHandle phid;
	CPhidgetList *traverse;
	
	if(!ANDROID_USB_GOOD)
		return EPHIDGET_UNSUPPORTED;
	
	result = (*ph_vm)->GetEnv(ph_vm, (void **)&env, JNI_VERSION_1_4);
	
	if(result == JNI_EDETACHED)
	{
		if ((*ph_vm)->AttachCurrentThread(ph_vm, (JNIEnvPtr)&env, NULL))
			JNI_ABORT_STDERR("Couldn't AttachCurrentThread");
	}
	
	phidgetList = (*env)->CallStaticObjectMethod(env, usb_phidget_class, usb_manager_getPhidgetList);
	if(phidgetList)
	{
		jsize phidgetListLength;
		int i, j, serial;
		
		phidgetListLength = (*env)->GetArrayLength(env, phidgetList);
		serial = phidgetListLength;
		
		LOG(PHIDGET_LOG_INFO, "In CPhidgetAndroidManager_getPhidgetList, length: %d", phidgetListLength);
		
		for(i=0;i<phidgetListLength;i++)
		{
			jobject phidget;
			phidget = (*env)->GetObjectArrayElement(env, phidgetList, i);

			if(phidget)
			{
				char uniqueName[1024];
				jstring str;
				jbyteArray labelArray;
				jbyte *datab;
				jsize len;
				int pID, vID;
				
				//Unique name - USBFS path
				str = (*env)->CallObjectMethod(env, phidget, com_phidgets_usb_Phidget_getUniqueName_mid);
				if(str)
				{
					jboolean iscopy;
					const char *textString = (*env)->GetStringUTFChars(env, str, &iscopy);
					strncpy(uniqueName, textString, 1024);
					(*env)->ReleaseStringUTFChars(env, str, textString);
				}
				else
				{
					res = EPHIDGET_UNEXPECTED;
					goto done;
				}
				
				//See if it's new
				LOG(PHIDGET_LOG_INFO, "Going to see if it's in AttachedDevices %d", AttachedDevices);
				if(AttachedDevices)
				{
					for(traverse = AttachedDevices; traverse; traverse = traverse->next)
					{
						if(!strcmp((char *)traverse->phid->CPhidgetFHandle, uniqueName))
						{
							CList_addToList((CListHandle *)curList, traverse->phid, CPhidget_areEqual);
							goto next;
						}
					}
				}
				
				//New, add it
				pID = (*env)->CallIntMethod(env, phidget, com_phidgets_usb_Phidget_getpID_mid);
				vID = (*env)->CallIntMethod(env, phidget, com_phidgets_usb_Phidget_getvID_mid);
				
				for (j = 1; j<PHIDGET_DEVICE_COUNT; j++)
				{
					if ((vID == Phid_Device_Def[j].pdd_vid) && (pID == Phid_Device_Def[j].pdd_pid))
					{
						if((res = CPhidget_create(&phid)) != EPHIDGET_OK)
							goto done;
						
						CPhidget_setStatusFlag(&phid->status, PHIDGET_ATTACHED_FLAG, &phid->lock);
						
						phid->deviceVersion = (*env)->CallIntMethod(env, phidget, com_phidgets_usb_Phidget_getVersion_mid);
						phid->deviceType = (char *)Phid_DeviceName[Phid_Device_Def[j].pdd_did];
						phid->deviceIDSpec = Phid_Device_Def[j].pdd_sdid;
						phid->deviceDef = &Phid_Device_Def[j];
						phid->deviceID = Phid_Device_Def[j].pdd_did;
						phid->ProductID = pID;
						phid->VendorID = vID;
						phid->serialNumber = (*env)->CallIntMethod(env, phidget, com_phidgets_usb_Phidget_getSerialNumber_mid);
						phid->specificDevice = TRUE;
						phid->attr = Phid_Device_Def[j].pdd_attr;
						
						//Label string
						labelArray = (*env)->CallObjectMethod(env, phidget, com_phidgets_usb_Phidget_getLabel_mid);
						if(labelArray)
						{
							len = (*env)->GetArrayLength(env, labelArray);
							datab = (*env)->GetByteArrayElements(env, labelArray, 0);
							if(datab)
							{
								res = decodeLabelString((char *)datab, phid->label, phid->serialNumber);
							}
							else
								phid->label[0]='\0';
							(*env)->ReleaseByteArrayElements(env, labelArray, datab, 0);
							if(res != EPHIDGET_OK)
								goto next;
						}
						else
						{
							res = EPHIDGET_UNEXPECTED;
							goto done;
						}
						
						if(!(phid->CPhidgetFHandle = strdup(uniqueName)))
						{
							res = EPHIDGET_NOMEMORY;
							goto done;
						}
						
						CList_addToList((CListHandle *)curList, phid, CPhidget_areEqual);
					}
				}
			} //if(phidget)
			else
			{
				res = EPHIDGET_UNEXPECTED;
				goto done;
			}
		next:;
		} //for(i=0;i<phidgetListLength;i++)
	}
	else
	{
		res = EPHIDGET_UNEXPECTED;
		phidgetList=NULL;
	}
	
done:
	(*ph_vm)->DetachCurrentThread(ph_vm);
	
	return res;
}
